/*
 Name:		STM32-ADC-Test.ino
 Created:	24.06.2022
 Author:	   Ulli
*/

// http://www.lucadavidian.com/2017/11/17/stm32f4-using-the-dma-controller-with-adc/
// https://controllerstech.com/dma-with-adc-using-registers-in-stm32/

#include <UrsUtil.h>
#include <UrsSTM32F411-fast-ADC.h>
#include "AppVersion.h"
#include "TestFunctions.h"


void setup() {
   Serial.begin();
   delay(500); // Warten, bis die Schnittstelle wieder angemeldet ist
   Serial.print("\n\n");
   Serial.print(APP_NAME); Serial.print(" Version "); Serial.println(VERSIONSTRING);

   Serial.printf("C++ Version: %s\n\n", UrsUtil.getCppVersion().c_str());




   uint32_t start, end;

   // =============================
   // Arduino analogRead
   // =============================
   // Test vorbereiten
   clear();
   analogReadResolution(12);

   start = micros();
   testArduinoAnalogRead();
   end = micros();
   reportSequence("Arduino analogRead", end - start, 8);

   // =============================
   // UrsADC analogRead
   // =============================
   // Test vorbereiten
   clear();

   UrsADC.enable();
   UrsADC.setResolution(UrsAdcResolution::Bit12);

   start = micros();
   testUrsAdcAnalogRead();
   end = micros();
   UrsADC.disable();
   reportSequence("UrsADC analogRead", end - start, 8);


   // =============================
   // UrsADC repeatRead 3 Cycles
   // =============================
   // Test vorbereiten
   clear();

   UrsADC.enable();
   UrsADC.setResolution(UrsAdcResolution::Bit12);
   UrsADC.setReadChannel(UrsAdcChannel::Channel2, UrsAdcSampleTime::Cycles_3);

   start = micros();
   testUrsAdcRead();
   end = micros();
   UrsADC.disable();
   reportSingleChannel("UrsADC repeatRead 3 Cycles", end - start, 20);


   // =============================
   // UrsADC repeatRead 15 Cycles
   // =============================
   // Test vorbereiten
   clear();

   UrsADC.enable();
   UrsADC.setResolution(UrsAdcResolution::Bit12);
   UrsADC.setReadChannel(UrsAdcChannel::Channel2, UrsAdcSampleTime::Cycles_15);

   start = micros();
   testUrsAdcRead();
   end = micros();
   UrsADC.disable();
   reportSingleChannel("UrsADC repeatRead 15 Cycles", end - start, 20);

   // =============================
   // UrsADC repeatRead 28 Cycles
   // =============================
   // Test vorbereiten
   clear();

   UrsADC.enable();
   UrsADC.setResolution(UrsAdcResolution::Bit12);
   UrsADC.setReadChannel(UrsAdcChannel::Channel2, UrsAdcSampleTime::Cycles_28);

   start = micros();
   testUrsAdcRead();
   end = micros();
   UrsADC.disable();
   reportSingleChannel("UrsADC repeatRead 28 Cycles", end - start, 20);


   // =============================
   // UrsADC repeatRead 56 Cycles
   // =============================
   // Test vorbereiten
   clear();

   UrsADC.enable();
   UrsADC.setResolution(UrsAdcResolution::Bit12);
   UrsADC.setReadChannel(UrsAdcChannel::Channel2, UrsAdcSampleTime::Cycles_56);

   start = micros();
   testUrsAdcRead();
   end = micros();
   UrsADC.disable();
   reportSingleChannel("UrsADC repeatRead 56 Cycles", end - start, 20);


   // =============================
   // UrsADC DMA Read 3 Cycles (Default)
   // =============================
   // Test vorbereiten
   clear();
   UrsADC.enable();
   UrsADC.setResolution(UrsAdcResolution::Bit12);

   start = micros();
   testUrsAdcDmaRead();
   end = micros();
   UrsADC.disable();
   reportSequence("UrsADC DMA read (3 cycles) ", end - start, 20);

   // =============================
   // UrsADC DMA Read 28 Cycles
   // =============================
   // Test vorbereiten
   clear();

   UrsADC.enable();
   UrsADC.setResolution(UrsAdcResolution::Bit12);
   UrsADC.setReadChannel(UrsAdcChannel::Channel0, UrsAdcSampleTime::Cycles_28);
   UrsADC.setReadChannel(UrsAdcChannel::Channel1, UrsAdcSampleTime::Cycles_28);
   UrsADC.setReadChannel(UrsAdcChannel::Channel2, UrsAdcSampleTime::Cycles_28);
   UrsADC.setReadChannel(UrsAdcChannel::Channel3, UrsAdcSampleTime::Cycles_28);
   UrsADC.setReadChannel(UrsAdcChannel::Channel4, UrsAdcSampleTime::Cycles_28);
   UrsADC.setReadChannel(UrsAdcChannel::Channel5, UrsAdcSampleTime::Cycles_28);
   UrsADC.setReadChannel(UrsAdcChannel::Channel6, UrsAdcSampleTime::Cycles_28);
   UrsADC.setReadChannel(UrsAdcChannel::Channel7, UrsAdcSampleTime::Cycles_28);

   start = micros();
   testUrsAdcDmaRead();
   end = micros();
   UrsADC.disable();
   reportSequence("UrsADC DMA read 28 Cycles", end - start, 20);

   // ==============================
   // UrsADC DMA Read 8 Bit 3 Cycles
   // ==============================
   // Test vorbereiten
   clear();

   UrsADC.enable();
   UrsADC.setResolution(UrsAdcResolution::Bit8);

   start = micros();
   testUrsAdcDmaRead();
   end = micros();
   UrsADC.disable();
   reportSequence8Bit("UrsADC DMA read 8 Bit (3 cycles) ", end - start, 20);


   // ==================================
   // UrsADC DMA Read mit ADC Interrupt
   // ==================================
   // Test vorbereiten
   clear();

   UrsADC.enable();
   UrsADC.setResolution(UrsAdcResolution::Bit12);

   // ADC auf sehr langsam einstellen, weil in den ISR zu Testzwecken Daten ausgegeben werden.
   UrsADC.setReadChannel(UrsAdcChannel::Channel0, UrsAdcSampleTime::Cycles_480);
   UrsADC.setReadChannel(UrsAdcChannel::Channel1, UrsAdcSampleTime::Cycles_480);
   UrsADC.setReadChannel(UrsAdcChannel::Channel2, UrsAdcSampleTime::Cycles_480);
   UrsADC.setReadChannel(UrsAdcChannel::Channel3, UrsAdcSampleTime::Cycles_480);
   UrsADC.setReadChannel(UrsAdcChannel::Channel4, UrsAdcSampleTime::Cycles_480);
   UrsADC.setReadChannel(UrsAdcChannel::Channel5, UrsAdcSampleTime::Cycles_480);
   UrsADC.setReadChannel(UrsAdcChannel::Channel6, UrsAdcSampleTime::Cycles_480);
   UrsADC.setReadChannel(UrsAdcChannel::Channel7, UrsAdcSampleTime::Cycles_480);
   UrsADC.setClockDivider(UrsAdcClockDivider::Div8);
   UrsADC.setEocSelection(UrsAdcEocSelection::Single);
   testUrsAdcDmaReadAdcIrq();

   UrsADC.disable();
   reportSequence("UrsADC DMA read 480 Cycles mit ADC Interrupt", 0, 10);

   Serial.println("Test-Ende");
}

void loop() {

}